@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2001 - 2003 Pekka Riikonen
+  Copyright (C) 2001 - 2007 Pekka Riikonen
 
   The contents of this file are subject to one of the Licenses specified 
   in the COPYING file;  You may not use this file except in compliance 
@@ -16,9 +16,9 @@
   information.
 
 */
-/* $Id: silcauth.c,v 1.28.2.4 2003/10/03 08:00:49 priikone Exp $ */
+/* $Id: silcauth.c,v 1.39 2007/01/23 14:52:01 priikone Exp $ */
 
-#include "silcincludes.h"
+#include "silc.h"
 #include "silcauth.h"
 
 /******************************************************************************
@@ -67,8 +67,8 @@ SilcAuthPayload silc_auth_payload_parse(const unsigned char *data,
     return NULL;
   }
 
-  if (newp->len != buffer.len ||
-      newp->random_len + newp->auth_len > buffer.len - 8) {
+  if (newp->len != silc_buffer_len(&buffer) ||
+      newp->random_len + newp->auth_len > silc_buffer_len(&buffer) - 8) {
     silc_auth_payload_free(newp);
     return NULL;
   }
@@ -201,24 +201,21 @@ silc_auth_public_key_encode_data(SilcPublicKey public_key,
 				 SilcIdType type, SilcUInt32 *ret_len)
 {
   SilcBuffer buf;
-  unsigned char *pk, *id_data, *ret;
+  unsigned char *pk, id_data[32], *ret;
   SilcUInt32 pk_len, id_len;
 
   pk = silc_pkcs_public_key_encode(public_key, &pk_len);
   if (!pk)
     return NULL;
 
-  id_data = silc_id_id2str(id, type);
-  if (!id_data) {
+  if (!silc_id_id2str(id, type, id_data, sizeof(id_data), &id_len)) {
     silc_free(pk);
     return NULL;
   }
-  id_len = silc_id_get_len(id, type);
 
   buf = silc_buffer_alloc_size(random_len + id_len + pk_len);
   if (!buf) {
     silc_free(pk);
-    silc_free(id_data);
     return NULL;
   }
   silc_buffer_format(buf,
@@ -230,7 +227,6 @@ silc_auth_public_key_encode_data(SilcPublicKey public_key,
   ret = silc_buffer_steal(buf, ret_len);
 
   silc_buffer_free(buf);
-  silc_free(id_data);
   silc_free(pk);
 
   return ret;
@@ -283,7 +279,6 @@ silc_auth_public_key_auth_generate_wpub(SilcPublicKey public_key,
   unsigned char *tmp;
   SilcUInt32 tmp_len;
   SilcBuffer buf;
-  SilcPKCS pkcs;
 
   SILC_LOG_DEBUG(("Generating Authentication Payload with data"));
 
@@ -293,22 +288,11 @@ silc_auth_public_key_auth_generate_wpub(SilcPublicKey public_key,
   if (!tmp)
     return NULL;
 
-  /* Allocate PKCS object */
-  if (!silc_pkcs_alloc(private_key->name, &pkcs)) {
-    memset(tmp, 0, tmp_len);
-    silc_free(tmp);
-    return NULL;
-  }
-  silc_pkcs_public_key_set(pkcs, public_key);
-  silc_pkcs_private_key_set(pkcs, private_key);
-
   /* Compute the hash and the signature. */
-  if (silc_pkcs_get_key_len(pkcs) / 8 > sizeof(auth_data) - 1 ||
-      !silc_pkcs_sign_with_hash(pkcs, hash, tmp, tmp_len, auth_data,
-				&auth_len)) {
+  if (!silc_pkcs_sign(private_key, tmp, tmp_len, auth_data,
+		      sizeof(auth_data) - 1, &auth_len, TRUE, hash)) {
     memset(tmp, 0, tmp_len);
     silc_free(tmp);
-    silc_pkcs_free(pkcs);
     return NULL;
   }
 
@@ -319,7 +303,6 @@ silc_auth_public_key_auth_generate_wpub(SilcPublicKey public_key,
   memset(tmp, 0, tmp_len);
   memset(auth_data, 0, sizeof(auth_data));
   silc_free(tmp);
-  silc_pkcs_free(pkcs);
 
   return buf;
 }
@@ -327,13 +310,13 @@ silc_auth_public_key_auth_generate_wpub(SilcPublicKey public_key,
 /* Verifies the authentication data. Returns TRUE if authentication was
    successful. */
 
-bool silc_auth_public_key_auth_verify(SilcAuthPayload payload,
-				      SilcPublicKey public_key, SilcHash hash,
-				      const void *id, SilcIdType type)
+SilcBool silc_auth_public_key_auth_verify(SilcAuthPayload payload,
+					  SilcPublicKey public_key,
+					  SilcHash hash,
+					  const void *id, SilcIdType type)
 {
   unsigned char *tmp;
   SilcUInt32 tmp_len;
-  SilcPKCS pkcs;
 
   SILC_LOG_DEBUG(("Verifying authentication data"));
 
@@ -346,28 +329,18 @@ bool silc_auth_public_key_auth_verify(SilcAuthPayload payload,
     return FALSE;
   }
 
-  /* Allocate PKCS object */
-  if (!silc_pkcs_alloc(public_key->name, &pkcs)) {
-    memset(tmp, 0, tmp_len);
-    silc_free(tmp);
-    return FALSE;
-  }
-  silc_pkcs_public_key_set(pkcs, public_key);
-
   /* Verify the authentication data */
-  if (!silc_pkcs_verify_with_hash(pkcs, hash, payload->auth_data,
-				  payload->auth_len, tmp, tmp_len)) {
+  if (!silc_pkcs_verify(public_key, payload->auth_data,
+			payload->auth_len, tmp, tmp_len, hash)) {
 
     memset(tmp, 0, tmp_len);
     silc_free(tmp);
-    silc_pkcs_free(pkcs);
     SILC_LOG_DEBUG(("Authentication failed"));
     return FALSE;
   }
 
   memset(tmp, 0, tmp_len);
   silc_free(tmp);
-  silc_pkcs_free(pkcs);
 
   SILC_LOG_DEBUG(("Authentication successful"));
 
@@ -376,11 +349,11 @@ bool silc_auth_public_key_auth_verify(SilcAuthPayload payload,
 
 /* Same as above but the payload is not parsed yet. This will parse it. */
 
-bool silc_auth_public_key_auth_verify_data(const unsigned char *payload,
-					   SilcUInt32 payload_len,
-					   SilcPublicKey public_key,
-					   SilcHash hash,
-					   const void *id, SilcIdType type)
+SilcBool silc_auth_public_key_auth_verify_data(const unsigned char *payload,
+					       SilcUInt32 payload_len,
+					       SilcPublicKey public_key,
+					       SilcHash hash,
+					       const void *id, SilcIdType type)
 {
   SilcAuthPayload auth_payload;
   int ret;
@@ -406,9 +379,9 @@ bool silc_auth_public_key_auth_verify_data(const unsigned char *payload,
    authentication then the `auth_data' is the SilcPublicKey and the
    `auth_data_len' is ignored. */
 
-bool silc_auth_verify(SilcAuthPayload payload, SilcAuthMethod auth_method,
-		      const void *auth_data, SilcUInt32 auth_data_len,
-		      SilcHash hash, const void *id, SilcIdType type)
+SilcBool silc_auth_verify(SilcAuthPayload payload, SilcAuthMethod auth_method,
+			  const void *auth_data, SilcUInt32 auth_data_len,
+			  SilcHash hash, const void *id, SilcIdType type)
 {
   SILC_LOG_DEBUG(("Verifying authentication"));
 
@@ -453,14 +426,15 @@ bool silc_auth_verify(SilcAuthPayload payload, SilcAuthMethod auth_method,
 
 /* Same as above but parses the authentication payload before verify. */
 
-bool silc_auth_verify_data(const unsigned char *payload,
-			   SilcUInt32 payload_len,
-			   SilcAuthMethod auth_method, const void *auth_data,
-			   SilcUInt32 auth_data_len, SilcHash hash,
-			   const void *id, SilcIdType type)
+SilcBool silc_auth_verify_data(const unsigned char *payload,
+			       SilcUInt32 payload_len,
+			       SilcAuthMethod auth_method,
+			       const void *auth_data,
+			       SilcUInt32 auth_data_len, SilcHash hash,
+			       const void *id, SilcIdType type)
 {
   SilcAuthPayload auth_payload;
-  bool ret;
+  SilcBool ret;
 
   auth_payload = silc_auth_payload_parse(payload, payload_len);
   if (!auth_payload || (auth_payload->auth_len == 0))
@@ -484,7 +458,8 @@ bool silc_auth_verify_data(const unsigned char *payload,
 struct SilcKeyAgreementPayloadStruct {
   SilcUInt16 hostname_len;
   unsigned char *hostname;
-  SilcUInt32 port;
+  SilcUInt16 protocol;
+  SilcUInt16 port;
 };
 
 /* Parses and returns an allocated Key Agreement payload. */
@@ -499,18 +474,19 @@ silc_key_agreement_payload_parse(const unsigned char *payload,
 
   SILC_LOG_DEBUG(("Parsing Key Agreement Payload"));
 
-  silc_buffer_set(&buffer, (unsigned char *)payload, payload_len);
   newp = silc_calloc(1, sizeof(*newp));
   if (!newp)
     return NULL;
 
   /* Parse the payload */
+  silc_buffer_set(&buffer, (unsigned char *)payload, payload_len);
   ret = silc_buffer_unformat(&buffer,
 			     SILC_STR_UI16_NSTRING_ALLOC(&newp->hostname,
 							 &newp->hostname_len),
-			     SILC_STR_UI_INT(&newp->port),
+			     SILC_STR_UI_SHORT(&newp->protocol),
+			     SILC_STR_UI_SHORT(&newp->port),
 			     SILC_STR_END);
-  if (ret == -1 || newp->hostname_len > buffer.len - 6) {
+  if (ret == -1 || newp->hostname_len > silc_buffer_len(&buffer) - 6) {
     silc_free(newp);
     return NULL;
   }
@@ -521,7 +497,8 @@ silc_key_agreement_payload_parse(const unsigned char *payload,
 /* Encodes the Key Agreement protocol and returns the encoded buffer */
 
 SilcBuffer silc_key_agreement_payload_encode(const char *hostname,
-					     SilcUInt32 port)
+					     SilcUInt16 protocol,
+					     SilcUInt16 port)
 {
   SilcBuffer buffer;
   SilcUInt32 len = hostname ? strlen(hostname) : 0;
@@ -534,7 +511,8 @@ SilcBuffer silc_key_agreement_payload_encode(const char *hostname,
   silc_buffer_format(buffer,
 		     SILC_STR_UI_SHORT(len),
 		     SILC_STR_UI_XNSTRING(hostname, len),
-		     SILC_STR_UI_INT(port),
+		     SILC_STR_UI_SHORT(protocol),
+		     SILC_STR_UI_SHORT(port),
 		     SILC_STR_END);
 
   return buffer;
@@ -557,9 +535,16 @@ char *silc_key_agreement_get_hostname(SilcKeyAgreementPayload payload)
   return payload->hostname;
 }
 
+/* Returns the protocol in the payload */
+
+SilcUInt16 silc_key_agreement_get_protocol(SilcKeyAgreementPayload payload)
+{
+  return payload->protocol;
+}
+
 /* Returns the port in the payload */
 
-SilcUInt32 silc_key_agreement_get_port(SilcKeyAgreementPayload payload)
+SilcUInt16 silc_key_agreement_get_port(SilcKeyAgreementPayload payload)
 {
   return payload->port;
 }
