@@ -1,6 +1,6 @@
 /* Stringprep tests */
 
-#include "silcincludes.h"
+#include "silc.h"
 
 typedef struct {
   const char *comment;
@@ -70,7 +70,7 @@ const test_st tests_norm[] = {
 
 int main(int argc, char **argv)
 {
-  bool success = FALSE;
+  SilcBool success = FALSE;
   int i, enc;
   unsigned char *out = NULL;
   SilcUInt32 out_len;
@@ -79,6 +79,7 @@ int main(int argc, char **argv)
   if (argc > 1 && !strcmp(argv[1], "-d")) {
     silc_log_debug(TRUE);
     silc_log_debug_hexdump(TRUE);
+    silc_log_quick(TRUE);
     silc_log_set_debug_string("*stringprep*,*utf8*");
   }
 
